@@ -1,8 +1,8 @@
 package Minion::Backend::MongoDB;
-$Minion::Backend::MongoDB::VERSION = '1.08';
+$Minion::Backend::MongoDB::VERSION = '1.09';
 # ABSTRACT: MongoDB backend for Minion
 
-use 5.016; # Minion requires this so we require this.
+use 5.016;    # Minion requires this so we require this.
 
 use Mojo::Base 'Minion::Backend';
 
@@ -35,7 +35,7 @@ sub broadcast {
       ( shift, shift, shift || [], shift || [] );
 
     my $match = {};
-    my @ids = map $s->_oid($_), @$ids;
+    my @ids   = map $s->_oid($_), @$ids;
     $match->{_id} = { '$in' => \@ids } if ( scalar(@ids) );
     my $res = $s->workers->update_many( $match,
         { '$push' => { inbox => [ $command, @$args ] } } );
@@ -47,19 +47,15 @@ sub dequeue {
     my ( $self, $id, $wait, $options ) = @_;
 
     if ( ( my $job = $self->_try( $id, $options ) ) ) { return $job }
-    my $l = Mojo::IOLoop->singleton;
-    return undef if $l->is_running;
+    return undef if Mojo::IOLoop->is_running;
 
-    # Capped collection for notifications
-    $self->_notifications;
-
-    my $timer = $l->timer( $wait => sub { $l->stop } );
+    my $timer = Mojo::IOLoop->timer( $wait => sub { Mojo::IOLoop->stop } );
     Mojo::Promise->new->resolve->then(
         sub() {
-            $l->stop if ( $self->_await );
+            Mojo::IOLoop->stop if ( $self->_await );
         }
     )->wait;
-    $l->remove($timer);
+    Mojo::IOLoop->remove($timer);
 
     return $self->_try( $id, $options );
 
@@ -69,9 +65,6 @@ sub enqueue {
     my ( $self, $task, $args, $options ) =
       ( shift, shift, shift || [], shift || {} );
 
-    # Capped collection for notifications
-    $self->_notifications;
-
     my $id;
 
     if ( my $seq = $options->{sequence} ) {
@@ -195,7 +188,7 @@ sub list_jobs {
         }
     };
 
-    # (select id from minion_jobs where sequence = j.sequence and next = j.id) as previous,
+# (select id from minion_jobs where sequence = j.sequence and next = j.id) as previous,
     my $l_previous = {
         '$lookup' => {
             from     => $self->prefix . '.jobs',
@@ -338,6 +331,15 @@ sub lock {
     return $s->_lock( $name, $duration, $options->{limit} || 1 );
 }
 
+sub minion {
+    my $s      = shift;
+    my $minion = shift;
+    return $s->{minion} unless $minion;
+    $s->{minion} = $minion;
+    $s->_reconnect_db;
+    return $s;
+}
+
 sub new {
     my ( $class, $url ) = ( shift, shift );
     my $client = MongoDB->connect( $url, @_ );
@@ -349,7 +351,6 @@ sub new {
             $self->mongodb->client->reconnect();
         }
     );
-
     return $self;
 }
 
@@ -410,6 +411,8 @@ sub receive {
 sub register_worker {
     my ( $self, $id, $options ) = @_;
 
+    $self->_init_db;
+
     return $id
       if $id
       && $self->workers->find_one_and_update(
@@ -422,18 +425,6 @@ sub register_worker {
         }
       );
 
-    # indexes for jobs
-    $self->jobs->indexes->create_one(
-        Tie::IxHash->new( state => 1, delayed => 1, task => 1, queue => 1 ) );
-    $self->jobs->indexes->create_one( Tie::IxHash->new( finished => 1 ) );
-    $self->jobs->indexes->create_one(
-        Tie::IxHash->new( sequence => 1, next => 1 ) );
-    $self->jobs->indexes->create_one( Tie::IxHash->new( expires => 1 ) );
-
-    # indexes for locks
-    $self->locks->indexes->create_one( Tie::IxHash->new( name => 1 ),
-        { unique => 1 } );
-    $self->locks->indexes->create_one( Tie::IxHash->new( expires => 1 ) );
     my $res = $self->workers->insert_one(
         {
             host     => hostname,
@@ -444,6 +435,7 @@ sub register_worker {
             inbox    => [],
         }
     );
+
     return $res->inserted_id->hex;
 }
 
@@ -476,7 +468,7 @@ sub repair {
 
     # Old jobs with no unresolved dependencies and expired jobs
 
-    # find: select 1 from minion_jobs where parents @> ARRAY[j.id] and state != 'finished'
+# find: select 1 from minion_jobs where parents @> ARRAY[j.id] and state != 'finished'
     my $docs = $jobs->aggregate(
         [
             {
@@ -513,7 +505,7 @@ sub repair {
     my @ids_to_delete;
     while ( my $doc = $docs->next ) {
         push @ids_to_delete, $doc->{_id}
-          unless ( scalar( $doc->{parents}->@* ) );
+          unless ( scalar( @{$doc->{parents}} ) );
     }
     $jobs->delete_many( { _id => { '$in' => \@ids_to_delete } } );
 
@@ -553,6 +545,7 @@ sub repair {
 
 sub reset {
     my ( $s, $options ) = ( shift, shift // {} );
+    $s->_init_db;
     if ( $options->{all} ) {
         $_->drop for $s->workers, $s->jobs, $s->locks;
     }
@@ -690,10 +683,13 @@ sub _enqueue {
     @{ $options->{parents} } = map $self->_oid($_), @{ $options->{parents} }
       if ( exists $options->{parents} );
 
+    my $now = DateTime->now();
     my $doc = {
-        args     => $args,
-        created  => DateTime->from_epoch( epoch => time ),
-        delayed  => DateTime->now()->add( seconds => $options->{delay} // 0 ),
+        args    => $args,
+        created => $now,
+        delayed => $options->{delay}
+        ? $now->clone->add( seconds => $options->{delay} )
+        : $now,
         priority => $options->{priority} // 0,
         state    => 'inactive',
         task     => $task,
@@ -703,7 +699,7 @@ sub _enqueue {
         parents  => $options->{parents} || [],
         queue    => $options->{queue} // 'default',
         sequence => $options->{sequence},
-        expires  => $options->{expire} ? DateTime->now()->add(
+        expires  => $options->{expire} ? $now->clone->add(
             seconds => $options->{expire}
         ) : undef,
         lax => $options->{lax} ? 1 : 0,
@@ -714,6 +710,26 @@ sub _enqueue {
     return $id;
 }
 
+sub _init_db {
+    my $s = shift;
+
+    # indexes for jobs
+    $s->jobs->indexes->create_one(
+        Tie::IxHash->new( state => 1, delayed => 1, task => 1, queue => 1 ) );
+    $s->jobs->indexes->create_one( Tie::IxHash->new( finished => 1 ) );
+    $s->jobs->indexes->create_one(
+        Tie::IxHash->new( sequence => 1, next => 1 ) );
+    $s->jobs->indexes->create_one( Tie::IxHash->new( expires => 1 ) );
+
+    # indexes for locks
+    $s->locks->indexes->create_one( Tie::IxHash->new( name => 1 ),
+        { unique => 1 } );
+    $s->locks->indexes->create_one( Tie::IxHash->new( expires => 1 ) );
+
+    # Capped collection for notifications
+    $s->_notifications;
+}
+
 sub _job_info {
     my $self = shift;
 
@@ -796,8 +812,8 @@ sub _notifications {
     $self->mongodb->run_command(
         [
             create => $notifications->name,
-            capped => 1,
-            size   => 1048576,
+            capped => true,
+            size   => 10240,
             max    => 128
         ]
     );
@@ -808,6 +824,23 @@ sub _oid {
     return defined $_[1] ? BSON::OID->new( oid => pack( "H*", $_[1] ) ) : undef;
 }
 
+sub _reconnect_db {
+    my $s = shift;
+    $s->minion->on(
+        worker => sub {
+            $_[1]->on(
+                dequeue => sub {
+                    $_[1]->on(
+                        start => sub {
+                            $s->minion->backend->mongodb->client->reconnect();
+                        }
+                    );
+                }
+            );
+        }
+    );
+}
+
 sub _total {
     my ( $name, $res, $tot ) = @_;
     return { total => $tot, $name => $res };
@@ -830,7 +863,8 @@ sub _try {
     $match->Push( 'priority' => { '$gte' => $options->{min_priority} } )
       if exists $options->{min_priority};
 
-    my $docs = $self->jobs->find($match)->sort( { priority => -1, id => 1 } );
+    my $docs =
+      $self->jobs->find( $match, { sort => [ 'priority', -1, '_id', 1 ] } );
 
     my $find = 0;
     my $doc_matched;
@@ -957,9 +991,17 @@ __END__
 
 Minion::Backend::MongoDB - MongoDB backend for Minion
 
+=for html <p>
+    <a href="https://github.com/avkhozov/Minion-Backend-MongoDB/actions/workflows/test.yml">
+        <img alt="github workflow tests" src="https://github.com/avkhozov/Minion-Backend-MongoDB/actions/workflows/test.yml/badge.svg">
+    </a>
+    <img alt="Top language: " src="https://img.shields.io/github/languages/top/avkhozov/Minion-Backend-MongoDB">
+    <img alt="github last commit" src="https://img.shields.io/github/last-commit/avkhozov/Minion-Backend-MongoDB">
+</p>
+
 =head1 VERSION
 
-version 1.08
+version 1.09
 
 =head1 SYNOPSIS
 
